==> jils-0.5.2.tgz: package/lib/evaluate.js <==
var env = require('./env'),
	core = require('./core');

module.exports = { evaluate: evaluate };

function evaluate(_, global){
	if (core.stringp(_)){
		try {
			if (_.indexOf('.') > -1){
				var paths = _.split('.'),
					root = env.eget(global, paths[0]),
					value = root;
				for (var n = 1, l = paths.length; n < l; n++)
					value = value[paths[n]];
				value.__root = root;
				return value;
			}
			return env.eget(global, _);
		} catch(e) {
			return _;
		}
	} else if (core.hashp(_)){
		var keys = env.keys(_),
			values = env.values(_).map(function(v){ 
				return evaluate(v, global);
			});
		return env.mkhash(keys,values);
	} else if (!core.arrayp(_)){
		return _;
	} else if (_.length == 0){
		return [];
	} else {
		switch (_[0]){
			case 'quote':
				return _[1];
			case 'hash':
				var value = evaluate(_[1], global),
					hash = {};
				value.map(function(kov,i){
					if (i % 2) hash[lastk] = kov;
					else lastk = kov.replace(/^:/,'');
				});
				return hash;
			case 'list':
				return _.slice(1).map(function(v){
					return evaluate(v, global);
				})
			case 'inst':
				var obj = evaluate(_[1], global);
				if (_[2] && _[length].length) return new obj(evaluate(_[2], global));
				else return new obj();
			case 'if':
				return evaluate(evaluate(_[1], global) ? _[2] : _[3], global);
			case 'def':
				var value = evaluate(_[2], global);
				return core.def(global, _[1], value);
			case 'loop':
				var cond = _[1], acc = [];
				while(evaluate(cond, global)){
					 acc.push(evaluate(_[2], global));
				}
				return acc;
			case 'assoc':
				var assign = core.assign(_[1],_[2]),
					keys = assign[0], values = assign[1];
				for (var i=0,l=keys.length;i<l;i++){
					core.def(global, keys[i], values[i]);
				} return _[2];
			case 'fn':
				var func = function(){
					return (function(keys, args){
						var assign = core.assign(keys, args),
							keys = assign[0], args = assign[1];
						return evaluate(_[2],
							env.mkenv(keys, args, global));
					})(_[1], core.slice(arguments));
				}; func.toString = function(){ return '[Function ('+_[1].join(' ')+')]' };
				return func;
			case 'do':
				var args = _.slice(1),
					result;
				for (var a in args)
					result = evaluate(args[a], global)
				return result;
			case 'eval':
				return evaluate(
					evaluate(_[1], global), 
				global);
			default:
				var proc = evaluate(_[0], global);
				if (proc.__macro){
					return evaluate(proc.apply(global, _.slice(1)), global); 
				}
				var exps = _.slice(1).map(function(exp){
					return evaluate(exp, global);
				});
				if (proc.__root){
					return proc.apply(proc.__root, exps);
				} return proc.apply && proc.apply(proc, exps);
		}
	}
}

==> jils-0.5.4.tgz: package/lib/evaluate.js <==
var env = require('./env'),
	core = require('./core');

module.exports = { evaluate: evaluate };

function evaluate(_, global){
	if (core.stringp(_)){
		try {
			if (_.indexOf('.') > -1){
				var paths = _.split('.'),
					root = env.eget(global, paths[0]),
					value = root;
				for (var n = 1, l = paths.length; n < l; n++)
					value = value[paths[n]];
				value.__root = root;
				return value;
			}
			return env.eget(global, _);
		} catch(e) {
			return _;
		}
	} else if (core.hashp(_)){
		var keys = env.keys(_),
			values = env.values(_).map(function(v){ 
				return evaluate(v, global);
			});
		return env.mkhash(keys,values);
	} else if (!core.arrayp(_)){
		return _;
	} else if (_.length == 0){
		return [];
	} else {
		switch (_[0]){
			case 'quote':
				return _[1];
			case 'hash':
				var value = evaluate(_[1], global),
					hash = {};
				value.map(function(kov,i){
					if (i % 2) hash[lastk] = kov;
					else lastk = kov.replace(/^:/,'');
				});
				return hash;
			case 'list':
				return _.slice(1).map(function(v){
					return evaluate(v, global);
				})
			case 'inst':
				var obj = evaluate(_[1], global);
				if (_[2] && _[length].length) return new obj(evaluate(_[2], global));
				else return new obj();
			case 'if':
				return evaluate(evaluate(_[1], global) ? _[2] : _[3], global);
			case 'def':
				var value = evaluate(_[2], global);
				return core.def(global, _[1], value);
			case 'loop':
				var cond = _[1], acc = [];
				while(evaluate(cond, global)){
					 acc.push(evaluate(_[2], global));
				}
				return acc;
			case 'assoc':
				var assign = core.assign(_[1],_[2]),
					keys = assign[0], values = assign[1];
				for (var i=0,l=keys.length;i<l;i++){
					core.def(global, keys[i], values[i]);
				} return _[2];
			case 'fn':
				var func = function(){
					return (function(keys, args){
						var assign = core.assign(keys, args),
							keys = assign[0], args = assign[1];
						return evaluate(_[2],
							env.mkenv(keys, args, global));
					})(_[1], core.slice(arguments));
				}; func.toString = function(){ return '[Function ('+_[1].join(' ')+')]' };
				return func;
			case 'do':
				var args = _.slice(1),
					result;
				for (var a in args)
					result = evaluate(args[a], global)
				return result;
			case 'eval':
				return evaluate(
					evaluate(_[1], global), 
				global);
			default:
				var proc = evaluate(_[0], global);
				if (proc.__macro){
					return evaluate(proc.apply(global, _.slice(1)), global); 
				}
				var exps = _.slice(1).map(function(exp){
					return evaluate(exp, global);
				});
				if (proc.__root){
					return proc.apply(proc.__root, exps);
				} return proc.apply && proc.apply(this, exps);
		}
	}
}
